(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@angular/core'), require('@angular/common/http'), require('rxjs'), require('crypto-js'), require('rxjs/add/operator/map')) :
    typeof define === 'function' && define.amd ? define('ngx-http-crypt', ['exports', '@angular/core', '@angular/common/http', 'rxjs', 'crypto-js', 'rxjs/add/operator/map'], factory) :
    (global = global || self, factory(global['ngx-http-crypt'] = {}, global.ng.core, global.ng.common.http, global.rxjs, global.CryptoJS));
}(this, (function (exports, core, http, rxjs, CryptoJS) { 'use strict';

    var CryptService = /** @class */ (function () {
        function CryptService() {
            this.keySize = 256;
            this.ivSize = 128;
            this.saltSize = 256;
            this.iterations = 1000;
        }
        CryptService.prototype.encrypt = function (msg, pass) {
            var salt = CryptoJS.lib.WordArray.random(this.saltSize / 8);
            var key = CryptoJS.PBKDF2(pass, salt, {
                keySize: this.keySize / 32,
                iterations: this.iterations
            });
            var iv = CryptoJS.lib.WordArray.random(this.ivSize / 8);
            var encrypted = CryptoJS.AES.encrypt(msg, key, {
                iv: iv,
                padding: CryptoJS.pad.Pkcs7,
                mode: CryptoJS.mode.CBC
            });
            var encryptedHex = this.base64ToHex(encrypted.toString());
            var base64result = this.hexToBase64(salt + iv + encryptedHex);
            return base64result;
        };
        CryptService.prototype.decrypt = function (transitmessage, pass) {
            if (transitmessage.toString() === '{}') {
                return JSON.stringify({});
            }
            var hexResult = this.base64ToHex(transitmessage);
            var salt = CryptoJS.enc.Hex.parse(hexResult.substr(0, 64));
            var iv = CryptoJS.enc.Hex.parse(hexResult.substr(64, 32));
            var encrypted = this.hexToBase64(hexResult.substring(96));
            var key = CryptoJS.PBKDF2(pass, salt, {
                keySize: this.keySize / 32,
                iterations: this.iterations
            });
            var decrypted = CryptoJS.AES.decrypt(encrypted, key, {
                iv: iv,
                padding: CryptoJS.pad.Pkcs7,
                mode: CryptoJS.mode.CBC
            });
            return decrypted.toString(CryptoJS.enc.Utf8);
        };
        CryptService.prototype.hexToBase64 = function (str) {
            return btoa(String.fromCharCode.apply(null, str.replace(/\r|\n/g, "").replace(/([\da-fA-F]{2}) ?/g, "0x$1 ").replace(/ +$/, "").split(" ")));
        };
        CryptService.prototype.base64ToHex = function (str) {
            for (var i = 0, bin = atob(str.replace(/[ \r\n]+$/, "")), hex = []; i < bin.length; ++i) {
                var tmp = bin.charCodeAt(i).toString(16);
                if (tmp.length === 1)
                    tmp = "0" + tmp;
                hex[hex.length] = tmp;
            }
            return hex.join("");
        };
        return CryptService;
    }());
    CryptService.decorators = [
        { type: core.Injectable }
    ];
    CryptService.ctorParameters = function () { return []; };

    var CryptHttpService = /** @class */ (function () {
        function CryptHttpService(http, crypt) {
            this.http = http;
            this.crypt = crypt;
            this.secret = '';
            this.isOnLoad = new rxjs.BehaviorSubject(this.running);
        }
        CryptHttpService.prototype.encrypt = function (content) {
            var ciphertext = this.crypt.encrypt(JSON.stringify(content), this.secret);
            return ciphertext;
        };
        CryptHttpService.prototype.decrypt = function (content) {
            if (!this.cryptInactive) {
                if (content.text) {
                    content = content.text();
                }
                var bytes = this.crypt.decrypt(content.toString(), this.secret);
                try {
                    var decrypted = JSON.parse(bytes);
                    this.isOnLoad.next(false);
                    return decrypted;
                }
                catch (err) {
                    this.isOnLoad.next(false);
                    return { error: 'Can not decypted the content' };
                }
            }
            this.isOnLoad.next(false);
            if (!this.warnInactive) {
                this.warn();
            }
            return content;
        };
        CryptHttpService.prototype.setCryptHeaders = function (headers) {
            if (!headers) {
                headers = new http.HttpHeaders();
            }
            headers = headers.append('Content-Type', 'application/text');
            headers = headers.append('Content-encrypted', 'aes');
            return headers;
        };
        CryptHttpService.prototype.warn = function () {
            console.warn('Warning: You transfer non-encrypted content to the interface. Please check your settings');
        };
        CryptHttpService.prototype.configure = function (options) {
            this.secret = options.secret;
            this.warnInactive = options.warnInactive;
            this.cryptInactive = options.cryptInactive;
        };
        CryptHttpService.prototype.onLoad = function () {
            return this.isOnLoad.asObservable();
        };
        CryptHttpService.prototype.get = function (url, options) {
            var _this = this;
            this.isOnLoad.next(true);
            if (!this.cryptInactive) {
                options.headers = this.setCryptHeaders(options.headers);
                options.responseType = 'text';
            }
            return this.http.get(url, options).map(function (res) {
                if (options && options.observe === 'response') {
                    var newRes = res;
                    newRes.body = _this.decrypt(newRes.body);
                    return newRes;
                }
                return _this.decrypt(res);
            }, function (e) { return _this.isOnLoad.next(false); });
        };
        CryptHttpService.prototype.patch = function (url, body, options) {
            var _this = this;
            this.isOnLoad.next(true);
            if (!this.cryptInactive) {
                options.headers = this.setCryptHeaders(options.headers);
                body = this.encrypt(body);
                options.responseType = 'text';
            }
            return this.http.patch(url, body, options).map(function (res) { return _this.decrypt(res); }, function (e) { return _this.isOnLoad.next(false); });
        };
        CryptHttpService.prototype.post = function (url, body, options) {
            var _this = this;
            this.isOnLoad.next(true);
            if (!this.cryptInactive) {
                options.headers = this.setCryptHeaders(options.headers);
                options.responseType = 'text';
                body = this.encrypt(body);
            }
            return this.http.post(url, body, options).map(function (res) { return _this.decrypt(res); }, function (e) { return _this.isOnLoad.next(false); });
        };
        CryptHttpService.prototype.put = function (url, body, options) {
            var _this = this;
            this.isOnLoad.next(true);
            if (!this.cryptInactive) {
                options.headers = this.setCryptHeaders(options.headers);
                body = this.encrypt(body);
                options.responseType = 'text';
            }
            return this.http.put(url, body, options).map(function (res) { return _this.decrypt(res); }, function (e) { return _this.isOnLoad.next(false); });
        };
        CryptHttpService.prototype.delete = function (url, options) {
            var _this = this;
            this.isOnLoad.next(true);
            if (!this.cryptInactive) {
                options.headers = this.setCryptHeaders(options.headers);
                options.responseType = 'text';
            }
            return this.http.delete(url, options).map(function (res) { return _this.decrypt(res); }, function (e) { return _this.isOnLoad.next(false); });
        };
        CryptHttpService.prototype.options = function (url, options) {
            var _this = this;
            this.isOnLoad.next(true);
            if (!this.cryptInactive) {
                options.headers = this.setCryptHeaders(options.headers);
                options.responseType = 'text';
            }
            return this.http.options(url, options).map(function (res) { return _this.decrypt(res); }, function (e) { return _this.isOnLoad.next(false); });
        };
        return CryptHttpService;
    }());
    CryptHttpService.decorators = [
        { type: core.Injectable }
    ];
    CryptHttpService.ctorParameters = function () { return [
        { type: http.HttpClient, decorators: [{ type: core.Inject, args: [http.HttpClient,] }] },
        { type: CryptService }
    ]; };

    var NgxHttpCryptModule = /** @class */ (function () {
        function NgxHttpCryptModule() {
        }
        return NgxHttpCryptModule;
    }());
    NgxHttpCryptModule.decorators = [
        { type: core.NgModule, args: [{
                    imports: [
                        http.HttpClientModule
                    ],
                    providers: [
                        CryptHttpService, CryptService
                    ]
                },] }
    ];

    /**
     * Generated bundle index. Do not edit.
     */

    exports.CryptHttpService = CryptHttpService;
    exports.CryptService = CryptService;
    exports.NgxHttpCryptModule = NgxHttpCryptModule;

    Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=ngx-http-crypt.umd.js.map
